import sys
import pandas
import os.path
from itertools import product
from Bio import SeqIO

# Reading the copied data from MicroSNiPer web-application output

microSNiPer_output = sys.argv[1]

with open(microSNiPer_output, 'r') as inpObj:
    lines = inpObj.readlines()

# Creating the variant class, describing a binding of a microRNA and 3'-UTR region with a mutation

class variant():
    def __init__(self):
        self.state = '' # Mutated or original allele of a variant
        self.posit = '' # Position of a variant in 3'-UTR sequence
        self.subst = '' # A certain substitute
        self.mirna = '' # MicroRNA that binds to a 3'-UTR region with a mutation
        self.seedl = '' # Seed length
        self.genen = '' # Gene name
        self.snpid = '' # SNP ID (rs...)

varList = list()

# Parsing the MicroSNiPer output

for l in lines:
    if 'POLYMORPHIC VARIANT' in l:
        if 'Original' in l:
            currState = 'original'
        else:
            currState = 'mutant'
    elif l.startswith('Gene:'):
        currGenen = l.split(' ')[1]
    elif l.startswith('SNP:'):
        if not ('-rs' in l):
            currSnpid = l.split(' ')[1]
            currPosit = l.split(' ')[2].split('[')[0]
            currSubst = l.split(' ')[2].split('[')[1].replace(']', '').replace('/', '>')
        else:
            raise Exception('Two mutations in a single query!!! Please, use only one variant at a time')
            currSnpid = l.split(' ')[1] + '-' + l.split('-')[1].split(' ')[0]
            currSubst = l.split('rs')[1].split(' ')[1] + l.split(' ')[-1]
    elif l.startswith('Seed'):
        currSeedl = l.split(' ')[2]
    elif 'nt)' in l:
        currMirna = l.split(' ')[0]
        v = variant()
        v.state = currState
        v.genen = currGenen
        v.snpid = currSnpid + ':' + currSubst.replace('\n', '')
        v.posit = int(currPosit)
        v.subst = currSubst
        v.seedl = int(currSeedl)
        v.mirna = currMirna
        varList.append(v)

# Creating a scaffold for a dataframe

data = { 
    'gene': [v.genen for v in varList],
    'snp_id': [v.snpid for v in varList],
    'substitution': [v.subst for v in varList],
    'posRNA': [v.posit for v in varList],
    'miRNA': [v.mirna for v in varList],
    'seed_length': [v.seedl for v in varList],
    'state': [v.state for v in varList]   
}

# Dataframe creation

df = pandas.DataFrame(data)
df['substitution'] = df['substitution'].str.replace('\n', '')

df.to_csv('microsniper.csv', index = False)

df = df.drop(columns = ['seed_length', 'miRNA', 'state'])
df = df.drop_duplicates()

# End of microSNiPer part

# Generating all pairs of miRNAs and mutations instead of microSNiPer

states = ['original', 'mutant']
with open('./db_mirna/mature_hsa.fa', 'r') as f:
    lines = f.readlines()
    
lines = [l for l in lines if l[0] == '>']
lines = [l[1:].split(' ')[0] for l in lines]
mirnas = lines

df1 = pandas.DataFrame(product(states, mirnas), columns = ['state', 'miRNA'])
df['key'] = 1
df1['key'] = 1
df = pandas.merge(df1, df, on='key').drop('key', axis=1)

utr_fasta = sys.argv[2]

utrFasta = str(SeqIO.read(utr_fasta, 'fasta').seq).upper().replace('T', 'U')

for index, row in df.iterrows():
    if (utrFasta == utrFasta[:row['posRNA']-1] + row['substitution'][0].replace('T','U') + utrFasta[row['posRNA']:]):
        pass
    else:
        print(utrFasta)
        print(row['substitution'][0].replace('T','U'))
        raise Exception(row)

# Generating a 50-nt regions of 3'-UTR sequence in the vicinity of each mutation

for index,row in df.iterrows():
    rightLimit = row['posRNA'] + 25
    if (row['posRNA'] + 25) >= len(utrFasta):
        rightLimit = len(utrFasta)
    leftLimit = row['posRNA'] - 25
    if (row['posRNA'] - 25) <= 0:
        leftLimit = 0
    if row['state'] == 'mutant':
        df.at[index, '3utr_seq'] = utrFasta[leftLimit:row['posRNA']-1] + row['substitution'][2].replace('T','U') + utrFasta[row['posRNA']:rightLimit]
    elif row['state'] == 'original':
        df.at[index, '3utr_seq'] = utrFasta[leftLimit:rightLimit]
    else:
        df.at[index, '3utr_seq'] = ''

df = df.drop(columns = ['gene'])

# All miRNAs - to a database

miRNADatabase = SeqIO.to_dict(SeqIO.parse('./db_mirna/mature_hsa.fa', 'fasta'))

for k, v in miRNADatabase.items():
    miRNADatabase[k] = str(v.seq)

# Incorporating miRNAs to a dataframe

for index, row in df.iterrows():
    df.at[index, 'miRNA_seq'] = miRNADatabase[row['miRNA']]

for index, row in df.iterrows():
    if miRNADatabase[row['miRNA']] != row['miRNA_seq']:
        raise Exception('miRNA sequence in table does not match its sequence in database')

for index, row in df.iterrows():
    if os.path.exists('./intarna/intarna_fastas/' + row['snp_id'] + '_' + row['state'] + '_3utr.fasta'):
        raise Exception('There are files in intarna_fasta directory. Remove them')

# Creating input files for IntaRNA

intarna_files = list()
for index, row in df.iterrows():
    utr = './intarna/intarna_fastas/{}_{}_3utr.fasta'.format(row['snp_id'], row['state'])
    intarna_files.append(utr)
    with open(utr, 'w') as f:
        f.write('>{0}_{1}\n{2}\n'.format(row['snp_id'], row['state'], row['3utr_seq']))

# Creating script for IntaRNA

with open('./intarna/intarna_script.sh', 'w') as sh:
    for utr in set(intarna_files):
        fasta = './db_mirna/intarna_mature_hsa.fa'
        sh.write('IntaRNA -t "{}" -q "{}" --outMode C --outCsvCols \'*\'\n'.format(utr, fasta))

df.to_csv('snps.csv', index=False)
